pub struct FlatDrop<K>(/* private fields */)
where
K: IntoOptionInner,
K::Inner: Recursive<Container = K>;Expand description
If K is a container of a recursive type, such as Box<T> where T: Recursive,
FlatDrop<K> behaves just like K, but with a custom Drop implementation.
In this implementation, we gather the recursive parts of the object iteratively
and drop them without recursion, avoiding stack overflows when dropping
large recursive objects.
Safety
We keep the invariant that the inner object is always initialised, but will
be dropped (exactly once) in the drop implementation.
Implementations
Sourceimpl<K> FlatDrop<K>
impl<K> FlatDrop<K>
pub const fn new(container: K) -> Self
pub fn into_inner(self) -> K