pub trait IntoOptionInner {
type Inner;
// Required method
fn into_option_inner(self) -> Option<Self::Inner>;
}Expand description
A trait for a smart pointer that contains (at most) a single value.
Required Associated Types
Required Methods
Sourcefn into_option_inner(self) -> Option<Self::Inner>
fn into_option_inner(self) -> Option<Self::Inner>
A (potentially) fallible operation to convert the container into its internal value. This should never drop any data.
If Self == Box, this will always return Some(*self).
If Self == Arc, this is Arc::into_inner.