Struct FlatDrop

pub struct FlatDrop<K>(/* private fields */)
where
    K: IntoOptionInner,
    K::Inner: Recursive<Container = K>;
If K is a container of a recursive type, such as Box<T> where T: Recursive, FlatDrop<K> behaves just like K, but with a custom Drop implementation. In this implementation, we gather the recursive parts of the object iteratively and drop them without recursion, avoiding stack overflows when dropping large recursive objects.

Safety

We keep the invariant that the inner object is always initialised, but will be dropped (exactly once) in the drop implementation.

Implementations

Source

impl<K> FlatDrop<K>
where K: IntoOptionInner, K::Inner: Recursive<Container = K>,

pub const fn new(container: K) -> Self

pub fn into_inner(self) -> K

Source

impl<T> FlatDrop<Box<T>>
where T: Recursive<Container = Box<T>>,

pub fn new_boxed(value: T) -> Self

Source

impl<T> FlatDrop<Rc<T>>
where T: Recursive<Container = Rc<T>>,

pub fn new_rc(value: T) -> Self

Source

impl<T> FlatDrop<Arc<T>>
where T: Recursive<Container = Arc<T>>,

pub fn new_arc(value: T) -> Self